package com.example.app.jmix.playground.playground;

import javax.sql.DataSource;

import org.slf4j.LoggerFactory;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.context.event.ApplicationStartedEvent;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.boot.autoconfigure.jdbc.DataSourceProperties;
import org.springframework.boot.context.properties.ConfigurationPropertiesScan;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Primary;
import org.springframework.context.event.EventListener;
import org.springframework.core.env.Environment;

import com.google.common.base.Strings;

@SpringBootApplication
@ConfigurationPropertiesScan({"com.example.app.jmix.playground"})
public class JmixPlaygroundApplication
{
    /**
     * Defines the <i>JMix</i> project identifier.
     */
    public static final String PROJECT_ID = "plgrnd";

    /**
     * Defines the <i>JMix</i> entity name prefix.
     */
    public static final String ENTITY_PREFIX = PROJECT_ID + '_';

    /**
     * Defines the <i>JMix</i> screen name prefix.
     */
    public static final String SCREEN_PREFIX = PROJECT_ID + '_';

    /**
     * Defines the <i>JMix</i> database table name prefix.
     */
    public static final String DB_TABLE_PREFIX = "PLGRND_";

    @Autowired
    private Environment environment;

    public static void main(final String[] someArgs)
    {
        SpringApplication.run(JmixPlaygroundApplication.class, someArgs);
    }

    @Bean
    @Primary
    @ConfigurationProperties("main.datasource")
    DataSourceProperties dataSourceProperties()
    {
        return new DataSourceProperties();
    }

    @Bean
    @Primary
    @ConfigurationProperties("main.datasource.hikari")
    DataSource dataSource(final DataSourceProperties someDataSourceProperties)
    {
        return someDataSourceProperties.initializeDataSourceBuilder().build();
    }

    @EventListener
    public void printApplicationUrl(final ApplicationStartedEvent anEvent)
    {
        LoggerFactory.getLogger(JmixPlaygroundApplication.class)
                        .info("Application started at " + "http://localhost:" + environment.getProperty(
                                        "local.server.port") + Strings.nullToEmpty(
                                        environment.getProperty("server.servlet.context-path")));
    }
}