package com.example.app.jmix.playground.playground.entity;

import java.util.Collection;
import java.util.Collections;
import java.util.UUID;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.Table;
import javax.persistence.Transient;
import javax.persistence.Version;
import javax.validation.constraints.Email;

import org.springframework.security.core.GrantedAuthority;

import com.example.app.jmix.playground.playground.JmixPlaygroundApplication;

import io.jmix.core.HasTimeZone;
import io.jmix.core.annotation.Secret;
import io.jmix.core.entity.annotation.JmixGeneratedValue;
import io.jmix.core.entity.annotation.SystemLevel;
import io.jmix.core.metamodel.annotation.DependsOnProperties;
import io.jmix.core.metamodel.annotation.InstanceName;
import io.jmix.core.metamodel.annotation.JmixEntity;
import io.jmix.security.authentication.JmixUserDetails;

@JmixEntity
@Entity(name = JmixPlaygroundApplication.ENTITY_PREFIX + "User")
@Table(name = User.TABLE_NAME, indexes = {
                @Index(name = "IDX_" + User.TABLE_NAME + "_ON_USERNAME", columnList = "USERNAME", unique = true) })
public class User implements JmixUserDetails, HasTimeZone
{
    /**
     * Defines the database table name.
     */
    public static final String TABLE_NAME = JmixPlaygroundApplication.DB_TABLE_PREFIX + "USER";

    /**
     * Generated serialization identifier.
     */
    private static final long serialVersionUID = 8737398748044353361L;

    @Id
    @Column(name = "ID")
    @JmixGeneratedValue
    private UUID id;

    @Version
    @Column(name = "VERSION", nullable = false)
    private Integer version;

    @Column(name = "USERNAME", nullable = false)
    protected String username;

    @Secret
    @SystemLevel
    @Column(name = "PASSWORD")
    protected String password;

    @Column(name = "FIRST_NAME")
    protected String firstName;

    @Column(name = "LAST_NAME")
    protected String lastName;

    @Email
    @Column(name = "EMAIL")
    protected String email;

    @Column(name = "ACTIVE")
    protected Boolean active = true;

    @Column(name = "TIME_ZONE_ID")
    protected String timeZoneId;

    @Transient
    protected Collection<? extends GrantedAuthority> authorities;

    public UUID getId()
    {
        return id;
    }

    public void setId(final UUID anId)
    {
        id = anId;
    }

    public Integer getVersion()
    {
        return version;
    }

    public void setVersion(final Integer aVersion)
    {
        version = aVersion;
    }

    public String getPassword()
    {
        return password;
    }

    @Override
    public String getUsername()
    {
        return username;
    }

    public void setUsername(final String aUsername)
    {
        username = aUsername;
    }

    public Boolean getActive()
    {
        return active;
    }

    public void setActive(final Boolean aFlag)
    {
        active = aFlag;
    }

    public void setPassword(final String aPassword)
    {
        password = aPassword;
    }

    public String getEmail()
    {
        return email;
    }

    public void setEmail(final String anEmail)
    {
        email = anEmail;
    }

    public String getFirstName()
    {
        return firstName;
    }

    public void setFirstName(final String aName)
    {
        firstName = aName;
    }

    public String getLastName()
    {
        return lastName;
    }

    public void setLastName(final String aName)
    {
        lastName = aName;
    }

    @Override
    public Collection<? extends GrantedAuthority> getAuthorities()
    {
        return authorities != null ? authorities : Collections.emptyList();
    }

    @Override
    public void setAuthorities(final Collection<? extends GrantedAuthority> someAuthorities)
    {
        authorities = someAuthorities;
    }

    @Override
    public boolean isAccountNonExpired()
    {
        return true;
    }

    @Override
    public boolean isAccountNonLocked()
    {
        return true;
    }

    @Override
    public boolean isCredentialsNonExpired()
    {
        return true;
    }

    @Override
    public boolean isEnabled()
    {
        return Boolean.TRUE.equals(active);
    }

    @InstanceName
    @DependsOnProperties({ "firstName", "lastName", "username" })
    public String getDisplayName()
    {
        return String.format("%s %s [%s]", (firstName == null ? "" : firstName), (lastName == null ? "" : lastName),
                        username).trim();
    }

    @Override
    public String getTimeZoneId()
    {
        return timeZoneId;
    }

    public void setTimeZoneId(final String aTimeZoneId)
    {
        timeZoneId = aTimeZoneId;
    }
}