package com.example.app.jmix.playground.playground.screen.customer;

import java.util.Objects;

import org.springframework.beans.factory.annotation.Autowired;

import io.jmix.ui.model.DataContext;
import io.jmix.ui.model.InstanceContainer;
import io.jmix.ui.screen.EditedEntityContainer;
import io.jmix.ui.screen.StandardEditor;
import io.jmix.ui.screen.Subscribe;
import io.jmix.ui.screen.UiController;
import io.jmix.ui.screen.UiDescriptor;

import com.example.app.jmix.playground.playground.entity.Context;
import com.example.app.jmix.playground.playground.entity.Customer;

@UiController("plgrnd_Customer.edit")
@UiDescriptor("customer-edit.xml")
@EditedEntityContainer("customerDc")
public class CustomerEdit extends StandardEditor<Customer>
{
    @Autowired
    private DataContext dataContext;

    @Autowired
    private InstanceContainer<Context> latestContextDc;

    private Context oldContext;

    @Subscribe
    protected void onBeforeShow(final BeforeShowEvent event)
    {
        initContext();
    }

    /**
     * Create a new {@link Context} and add this to the edited customer.
     */
    private void initContext()
    {
        oldContext = getEditedEntity().getLatestContext();

        final Context trackedContext = dataContext.create(Context.class);

        latestContextDc.setItem(trackedContext);
        if (oldContext != null)
        {
            trackedContext.setUser(oldContext.getUser());
        }
        trackedContext.setCustomer(getEditedEntity());
    }

    /**
     * <p>Compares the context created before show with the old "current context".</p>
     * <p>Removes the context from {@link DataContext} if it's unchanged.</p>
     * <p>Creates a second inactive context if old context is null and new is valid from future.</p>
     *
     * @param event
     *                 A before commit changes event.
     */
    @Subscribe
    protected void onBeforeCommitChanges(final BeforeCommitChangesEvent event)
    {
        final Context newContext = latestContextDc.getItemOrNull();

        // Should not happen!
        if (newContext == null)
        {
            event.preventCommit();
            return;
        }

        // If properties in context didn't change,...
        if (oldContext != null && Objects.equals(oldContext.getUser(), newContext.getUser()))
        {
            dataContext.remove(newContext);
        }
        event.resume();
    }
}