package com.example.app.jmix.playground.playground.screen.user;

import java.util.Arrays;
import java.util.Objects;
import java.util.TimeZone;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.crypto.password.PasswordEncoder;

import com.example.app.jmix.playground.playground.JmixPlaygroundApplication;
import com.example.app.jmix.playground.playground.entity.User;

import io.jmix.core.EntityStates;
import io.jmix.ui.Notifications;
import io.jmix.ui.action.list.EditAction;
import io.jmix.ui.component.ComboBox;
import io.jmix.ui.component.PasswordField;
import io.jmix.ui.component.TextField;
import io.jmix.ui.navigation.Route;
import io.jmix.ui.screen.EditedEntityContainer;
import io.jmix.ui.screen.MessageBundle;
import io.jmix.ui.screen.StandardEditor;
import io.jmix.ui.screen.Subscribe;
import io.jmix.ui.screen.UiController;
import io.jmix.ui.screen.UiDescriptor;

@UiController(UserEdit.UI_CONTROLLER)
@UiDescriptor(UserEdit.UI_DESCRIPTOR)
@EditedEntityContainer(UserEdit.DC_EDITED_ENTITY)
@Route(value = UserEdit.ROUTE_PATH, parentPrefix = UserEdit.ROUTE_PARENT_PREFIX)
public class UserEdit extends StandardEditor<User>
{
    /**
     * Defines the edited entity container name.
     */
    public static final String DC_EDITED_ENTITY = "userDc";

    /**
     * Defines the navigation route path.
     */
    public static final String ROUTE_PATH = "users/edit";

    /**
     * Defines the navigation route path.
     */
    public static final String ROUTE_PARENT_PREFIX = "users";

    /**
     * Defines the UI controller name.
     */
    public static final String UI_CONTROLLER = JmixPlaygroundApplication.SCREEN_PREFIX + "User." + EditAction.ID;

    /**
     * Defines the UI descriptor file name.
     */
    public static final String UI_DESCRIPTOR = "user-edit.xml";

    @Autowired
    private EntityStates entityStates;

    @Autowired
    private Notifications notifications;

    @Autowired
    private MessageBundle messageBundle;

    @Autowired
    private PasswordEncoder passwordEncoder;

    @Autowired
    private PasswordField passwordField;

    @Autowired
    private TextField<String> usernameField;

    @Autowired
    private PasswordField confirmPasswordField;

    @Autowired
    private ComboBox<String> timeZoneField;

    @Subscribe
    public void onInitEntity(final InitEntityEvent<User> anEvent)
    {
        usernameField.setEditable(true);
        passwordField.setVisible(true);
        confirmPasswordField.setVisible(true);
    }

    @Subscribe
    public void onAfterShow(final AfterShowEvent anEvent)
    {
        if (entityStates.isNew(getEditedEntity()))
        {
            usernameField.focus();
        }
    }

    @Subscribe
    protected void onBeforeCommit(final BeforeCommitChangesEvent anEvent)
    {
        if (entityStates.isNew(getEditedEntity()))
        {
            if (!Objects.equals(passwordField.getValue(), confirmPasswordField.getValue()))
            {
                notifications.create(Notifications.NotificationType.WARNING)
                                .withCaption(messageBundle.getMessage("passwordsDoNotMatch")).show();
                anEvent.preventCommit();
            }
            getEditedEntity().setPassword(passwordEncoder.encode(passwordField.getValue()));
        }
    }

    @Subscribe
    public void onInit(final InitEvent anEvent)
    {
        timeZoneField.setOptionsList(Arrays.asList(TimeZone.getAvailableIDs()));
    }
}