package com.example.app.jmix.playground.playground.user;

import java.util.Optional;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.junit.runner.RunWith;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import io.jmix.core.DataManager;
import io.jmix.core.security.SystemAuthenticator;

import com.example.app.jmix.playground.playground.JmixPlaygroundApplication;
import com.example.app.jmix.playground.playground.entity.User;

@RunWith(SpringJUnit4ClassRunner.class)
@SpringBootTest(classes = JmixPlaygroundApplication.class)
public class BasicUserTest
{
    /**
     * Manages data access. We'll need this to create some test data.
     */
    @Autowired
    DataManager dataManager;

    /**
     * Holds autowired system authenticator for authentication. We need an authentication for data access.
     */
    @Autowired
    SystemAuthenticator systemAuthenticator;

    /**
     * Class specific logger.
     */
    private Logger logger;

    /**
     * Sets up an authentication before each test.
     */
    @BeforeEach
    void setUp()
    {
        logger = LoggerFactory.getLogger(BasicUserTest.class);

        systemAuthenticator.begin();
    }

    /**
     * Tears authentication down after each test.
     */
    @AfterEach
    void tearDown()
    {
        systemAuthenticator.end();
    }

    /**
     * Test example from JMix forum.
     * {@see https://forum.jmix.io/t/document-about-testing/309}
     */
    @Test
    @DisplayName("test admin user exists")
    void adminExists()
    {
        final String query = "e.username = ?1";

        final Optional<User> adminOpt = dataManager.load(User.class)
                        .query(query, "admin")
                        .optional();

        logger.info("Admin user present: {}", adminOpt.isPresent());
        if (adminOpt.isPresent())
        {
            final User admin = adminOpt.get();
            logger.info("Admin user: {}", admin);
        }

        Assertions.assertTrue(adminOpt.isPresent(), "admin present");
    }
}
